//	*******************************************************************************
//	Name	:	S_Exec.hxx
//			:	ShellExecute
//	Author	:	I. Nakagawa
//	Create	:	2016/10/13
//	Modify	:	2018/05/11	add "explorer.exe"
//	Modify	:	2018/12/17	explorer_...	-->	static
//	Modify	:	2021/05/14	add <Windows.h>
//	Modify	:	2024/01/25	add ::S_Exec__explorer
//	Modify	:	2024/03/04	add ::Path_Normalize
//	Modify	:	2024/11/27
//	*******************************************************************************

#pragma		once

#include	<Windows.h>
#include	<ShellAPI.h>
//#include	"i_SH_Get.hxx"
#include	"SH_GetF_.hxx"
//#include	"i_trace.hxx"
#include	"filepath.hxx"
#include	"existff.hxx"
#include	"lgmsgbox.hxx"

////
//	*******************************************************************************
//	::ShellExecute			shellapi.h	shell32.lib
//	Create	:	2016/10/13
//	Modify	:	2018/05/11	add explorer
//	*******************************************************************************
class	S_Exec	{
public:
					S_Exec			() ;
public:
	virtual	bool	SetShowCmd		(const int showCmd)		{	ShowCmd = showCmd ;		return	true ;	}
	virtual	bool	SetOperation	(LPCTSTR operation)		{
					if		(operation == _T("open"))		{	Operation = operation ;		}
					else if	(operation == _T("print"))		{	Operation = operation ;		}
					else if	(operation == _T("explore"))	{	Operation = operation ;		}
					else if	(operation == _T("edit"))		{	Operation = operation ;		}
					else if	(operation == _T("find"))		{	Operation = operation ;		}
					else									{	Operation = operation ;		}
					LpOperation = Operation.c_str() ;
					return	true ;
					}
	virtual	bool	SetFile			(LPCTSTR file)			{
					if (::File_IsExist(file))	{	File = file ;				}
					else						{	File = GetPathEXE(file) ;	}
					LpFile = File.c_str() ;
					return	true ;
					}
	virtual	bool	SetParamaters	(LPCTSTR paramaters)	{
			//		if (LpFile == NULL)	{	return	SetFile(paramaters) ;	}
					Paramaters = paramaters ;
					LpParamaters = Paramaters.c_str() ;
					return	true ;
					}
public:
	virtual	bool	Execute			(void)	const ;
//	virtual	bool	Print			(void)	const ;
public:
	static	tstring	GetPathEXE	(LPCTSTR defPath) ;
	static	bool	IsExistEXE	(LPCTSTR defPath) ;
protected:
	HWND			HWnd ;
	int				ShowCmd ;
	LPCTSTR			LpOperation ;
	LPCTSTR			LpFile ;
	LPCTSTR			LpParamaters ;
	LPCTSTR			LpDirectory ;
	tstring			Operation ;
	tstring			File ;
	tstring			Paramaters ;
	tstring			Directory ;
protected:
//	HINSTANCE		HInstance ;
public:
	//	*******************************************************************************
	//	explorer
	//	Create	:	2018/05/10
	//	Modify	:	2018/12/17	static
	//	*******************************************************************************
	virtual	bool	explorer_	(LPCTSTR path)	{
					SetFile(_T("explorer.exe")) ;
					SetParamaters(path) ;
					return	Execute() ;
					}
//	virtual	bool	explorer_s	(LPCTSTR path)	{				return	  explorer ((tstring(_T(   "/select,"))+path).c_str()) ;	}
//	virtual	bool	explorer_es	(LPCTSTR path)	{				return	  explorer ((tstring(_T("/e,/select,"))+path).c_str()) ;	}
//	static	bool	explorer	(LPCTSTR path)	{	S_Exec	s ;	return	s.explorer_( 						    path		 ) ;	}
//	static	bool	explorer_s	(LPCTSTR path)	{	S_Exec	s ;	return	s.explorer_((tstring(_T(   "/select,"))+path).c_str()) ;	}
//	static	bool	explorer_es	(LPCTSTR path)	{	S_Exec	s ;	return	s.explorer_((tstring(_T("/e,/select,"))+path).c_str()) ;	}
protected:
	//	*******************************************************************************
	//	path normalize
	//	Create	:	2024/03/04
	//	*******************************************************************************
	static	tstring	Normalize	(LPCTSTR path_)	{
					tstring	path = path_ ;
					if (::File_IsExist(path)) {
						path = ::Path_Normalize(path) ;
						}
					return	path ;
					}
	//	*******************************************************************************
	//	explorer
	//	Create	:	2018/05/10
	//	Modify	:	2018/12/17	static
	//	Modify	:	2024/03/04	add ::Path_Normalize
	//	*******************************************************************************
public:
	static	bool	explorer	(LPCTSTR path)	{	S_Exec	s ;	return	s.explorer_( 				   Normalize(path) .c_str()) ;	}
	static	bool	explorer_s	(LPCTSTR path)	{	S_Exec	s ;	return	s.explorer_((_T(   "/select,")+Normalize(path)).c_str()) ;	}
	static	bool	explorer_es	(LPCTSTR path)	{	S_Exec	s ;	return	s.explorer_((_T("/e,/select,")+Normalize(path)).c_str()) ;	}
	} ;

////
//	*******************************************************************************
//	S_Execute
//	Create	:	2016/10/17
//	*******************************************************************************
inline	S_Exec::S_Exec()
{
	HWnd			= NULL ;
	LpOperation 	= NULL ;
	LpFile			= NULL ;
	LpParamaters	= NULL ;
	LpDirectory		= NULL ;
	ShowCmd			= SW_SHOWNORMAL ;
//	#ifdef	_MFC_VER
	#ifdef	__AFXWIN_H__
	{
		CWnd*	mainWnd = AfxGetMainWnd() ;
		if (mainWnd != NULL) {
			HWnd	= mainWnd->GetSafeHwnd() ;
			}
		}
	#endif
//	#endif
	}

////
//	*******************************************************************************
//	call ShellExecute
//	Create	:	2016/10/17
//	*******************************************************************************
inline	bool	S_Exec::Execute	(void)	const
{
	HINSTANCE	hInstance = 0 ;
	if (LpFile == NULL)	{	hInstance = ::ShellExecute(HWnd,LpOperation,LpParamaters,NULL,	LpDirectory,ShowCmd) ;	}
	else				{	hInstance = ::ShellExecute(HWnd,LpOperation,LpFile,LpParamaters,LpDirectory,ShowCmd) ;	}
	if (UINT64(hInstance) > 32)	{	return	true ;	}
	tstring	str ;
	switch	(UINT64(hInstance))	{
		case	0 :						str =	_T("The operating system is out of memory or resources.") ;				break ;
		case	ERROR_FILE_NOT_FOUND :	str =	_T("The specified file was not found.") ;								break ;
		case	ERROR_PATH_NOT_FOUND :	str =	_T("The specified path was not found.") ;								break ;
		case	ERROR_BAD_FORMAT :		str =
				_T("The .EXE file is invalid (non-Win32 .EXE or error in .EXE image).") ;								break ;
		case	SE_ERR_ACCESSDENIED :	str =	_T("The operating system denied access to the specified file.") ;		break ;
		case	SE_ERR_ASSOCINCOMPLETE:	str =	_T("The filename association is incomplete or invalid.") ;				break ;
		case	SE_ERR_DDEBUSY :		str =
				_T("The DDE transaction could not be completed because other DDE transactions were being processed.") ;	break ;
		case	SE_ERR_DDEFAIL :		str =	_T("The DDE transaction failed.") ;										break ;
		case	SE_ERR_DDETIMEOUT :		str =
				_T("The DDE transaction could not be completed because the request timed out.") ;						break ;
		case	SE_ERR_DLLNOTFOUND :	str =	_T("The specified dynamic-link library was not found.") ;				break ;
	//	case	SE_ERR_FNF :			str =	_T("The specified file was not found.") ;								break ;
		case	SE_ERR_NOASSOC :		str =
				_T("There is no application associated with the given file name extension.") ;							break ;
		case	SE_ERR_OOM :			str =	_T("There was not enough memory to complete the operation.") ;			break ;
		case	SE_ERR_SHARE :			str =	_T("A sharing violation occurred.") ;									break ;
	//	default :						str.Format(_T("Error(%s,%s)\n,hInstance=%d"),LpFile,LpParamaters,hInstance) ;	break ;
	//	default :						str.Format(_T("Error(%s,%s)\n"),			 LpFile,LpParamaters) ;				break ;
		default :						str =	_T("Error\n")	+	File	+	_T("\n")	+	Paramaters ;			break ;
		}
	str += _T("\nOperation=")	+ ::QuotM_Add(Operation) ;
	str += _T("\nFile=")		+ ::QuotM_Add(File) ;
	str += _T("\nParamaters=")	+ ::QuotM_Add(Paramaters) ;
	::LogMessageBox(str) ;
	return	false ;
	}


////
//	*******************************************************************************
//	get exe path		Program Files , ... (x86)	,	Windows , System
//				"Windows NT\Accessories\WordPad.exe"
//				"NotePad.exe"
//	Create	:	2016/10/13
//	*******************************************************************************
inline	tstring	S_Exec::GetPathEXE	(LPCTSTR defPath)
{
	tstring	progPath= ::SH_GetFolderPath(CSIDL_PROGRAM_FILES) ;
	tstring	exePath = progPath ;
			exePath = ::Path_AddLastSP(exePath) + defPath ;
	if (::File_IsExist(defPath))				{	exePath = defPath ;			}	//	defPath is full path
	if (::File_IsNothing(exePath)) {
		tstring	prog64exePath = tstring(_T("C:\\Program Files\\"))		+ defPath ;
		tstring	prog86exePath = tstring(_T("C:\\Program Files (x86)\\"))+ defPath ;
		if		(::File_IsExist(prog64exePath))	{	exePath = prog64exePath ;	}
		else if	(::File_IsExist(prog86exePath))	{	exePath = prog86exePath ;	}
		}
	if (::File_IsExist(exePath)) {
		return	exePath ;
		}
	if (::Path_GetExtLow(defPath) == _T("exe")) {
		tstring	exeName = defPath ;
				exeName = ::Path_GetName(defPath) ;
		tstring	winPath = ::SH_GetFolderPath(CSIDL_WINDOWS) ;
		tstring	sysPath = ::SH_GetFolderPath(CSIDL_SYSTEM) ;
		tstring winName = ::Path_AddLastSP(winPath) + exeName ;
		tstring	sysName = ::Path_AddLastSP(sysPath) + exeName ;
		if		(::File_IsExist(winName))		{	exePath = winName ;		}
		else if	(::File_IsExist(sysName))		{	exePath = sysName ;		}
		}
	if (::File_IsExist(exePath)) {
		return	exePath ;
		}
	return	defPath ;
	}

inline	bool	S_Exec::IsExistEXE	(LPCTSTR defPath)
{
	return	(::File_IsExist(S_Exec::GetPathEXE(defPath))) ;
	}

////
//	*******************************************************************************
//	explorer
//	Create	:	2024/01/25
//	*******************************************************************************
inline	bool	S_Exec__explorer(LPCTSTR path)
{
	static	v_tstring	opend_path ;
	bool	is_opend = false ;
	for (size_t index=0 ; index<opend_path.size() ; index++) {
		tstring	o_path = opend_path[index] ;
		if (path == o_path) {
			is_opend = true ;
			break ;
			}
		}
	if (!is_opend) {
		S_Exec::explorer(path) ;
		opend_path.push_back(path) ;
		}
	return	true ;
	}

