//	*******************************************************************************
//	Name	:	get_path.hxx
//			:	Get Current Directory ,	Temp Path
//	Author	:	I. Nakagawa
//	Create	:	2013/08/27
//	Modify	:	2017/07/18		for DS115j
//	Modify	:	2021/07/06		add ::Get_module_name
//	*******************************************************************************

#pragma		once

#include	<cstdlib>
#include	"tstring.hxx"
#include	"existff.hxx"
#include	"createff.hxx"
#include	"cmd_line.hxx"

#ifdef	_WIN32
	#include	<Windows.h>
	#include	"SH_GetF_.hxx"
#endif
#ifdef	__GNUC__
	#include	<unistd.h>
#endif

#ifdef	MAX_PATH
	#define	i_MAX_PATH	MAX_PATH
#else
	#define	i_MAX_PATH	260
#endif

/*
#ifdef		Tmp_Product
#else
	#ifdef	PIR_Reg_Product
			#define	Tmp_Product		PIR_Reg_Product
	#else
		#ifdef	_MSC_VER
		#else
		#pragma	message	("You must define \"Tmp_Product\" or \"PIR_Reg_Product\"  before Get_Path.hxx")
		#endif
		#ifdef	_DEBUG
			#define	Tmp_Product		_T("use PIR_Reg_Product")
		#else
			#define	Tmp_Product		_T("undefined")
		#endif
	#endif
#endif
*/

////
//	*******************************************************************************
//	Get Current Directory
//	Create	:	2013/08/27
//	*******************************************************************************
inline	tstring	GetCWD	(void)
{
	tstring	cwd ;
	{
		tstring	curName ;
		size_t	size = i_MAX_PATH ;
		#ifdef	_WIN32
			#ifndef _WIN32_WCE
			{
				curName.resize(size,0) ;
				::GetCurrentDirectory	(DWORD(size),&curName[0]) ;
				}
			#endif
		#endif
		#ifdef	__GNUC__
			{
				curName.resize(size,0) ;
				::getcwd	(&curName[0],size) ;
				}
		#endif
		cwd = curName.c_str() ;
		}
	return	cwd ;
	}

//	*******************************************************************************
//	Get Long Path Name
//	Create	:	2013/07/31
//	*******************************************************************************
inline	tstring	GetLngPathName	(c_tstring& fileName)
{
	tstring	longPath = fileName ;
	#ifdef	_MSC_VER
	#ifndef _WIN32_WCE
	{
		size_t	size = i_MAX_PATH ;
		longPath.resize(size,0) ;
		DWORD	len  = ::GetLongPathName(fileName.c_str(),&longPath[0],DWORD(size)) ;
		if (len == 0) {	return	fileName ;	}
		}
	#else
	#endif
	#endif
	return	longPath ;
	}

////
//	*******************************************************************************
//	Get Temp Path
//	Create	:	2013/08/27
//	*******************************************************************************
inline	tstring	GetTmpPath	(void)
{
	tstring	tmpPath ;
	{
		tstring	tmpName ;
		#ifdef	_WIN32
			{
				size_t	size = i_MAX_PATH ;
				tmpName.resize(size,0) ;
				::GetTempPath(DWORD(size),&tmpName[0]) ;
				tmpName = ::GetLngPathName(tmpName) ;
				}
			{
				if (::String_FindNoCase(tmpName,_T("Windows")) != tstring::npos) {
					tmpName = ::SH_GetFolderPath(CSIDL_COMMON_DOCUMENTS) ;
					tmpName = ::Path_AddLastSP(tmpName) + _T("Temp") ;
					tmpName = ::Path_AddLastSP(tmpName) + _T("Temp") ;
					}
				}
		#endif
		if (tmpName.empty()) {
			#define	E_Tmp1  	_T("TMP")
			#define	E_Tmp2  	_T("TMPDIR")
			#define	E_Tmp3  	_T("TEMP")
			#define	E_Usr1  	_T("USER")
			#define	E_Usr2  	_T("USERNAME")
			tstring	eTmp ;
		//	tstring	vTmp ;
		//	if (eTmp.empty())	{	eTmp = ::TGetEnv(E_Tmp1) ;	if (eTmp.empty())	{	vTmp = E_Tmp1 ;	}	}
		//	if (eTmp.empty())	{	eTmp = ::TGetEnv(E_Tmp2) ;	if (eTmp.empty())	{	vTmp = E_Tmp2 ;	}	}
		//	if (eTmp.empty())	{	eTmp = ::TGetEnv(E_Tmp3) ;	if (eTmp.empty())	{	vTmp = E_Tmp3 ;	}	}
			if (eTmp.empty())	{	eTmp = ::TGetEnv(E_Tmp1) ;		}
			if (eTmp.empty())	{	eTmp = ::TGetEnv(E_Tmp2) ;		}
			if (eTmp.empty())	{	eTmp = ::TGetEnv(E_Tmp3) ;		}
			if (!eTmp.empty())	{	tmpName = eTmp ;				}
			else {
				tstring	tmp_prdct ;
				{
					tstring	eUsr ;
					if (eUsr.empty())	{	eUsr = ::TGetEnv(E_Usr1) ;		}
					if (eUsr.empty())	{	eUsr = ::TGetEnv(E_Usr2) ;		}
					if (eUsr.empty())	{	eUsr = _T("___") ;				}
					if (::Folder_IsExist(_T("/tmp")))	{	tmp_prdct = _T("/tmp/") ;	}
					else								{	tmp_prdct = ::GetCWD() ;	}
				//	tmp_prdct = ::Path_AddLastSP(tmp_prdct) + Tmp_Product ;
					tmp_prdct = ::Path_AddLastSP(tmp_prdct) + eUsr ;
					}
				::Folder_Create(tmp_prdct) ;
				tmpName = tmp_prdct ;
				}
			}
		tmpPath = tmpName.c_str() ;
		}
	return	tmpPath ;
	}

////
//	*******************************************************************************
//	get absolute path
//	Create	:	2021/07/06
//	*******************************************************************************
inline	tstring	Get_abs_path	(LPCTSTR path_)
{
	tstring	path = path_ ;
	#ifdef	_WIN32
	#else
	{
		if (path.length() > 0) {
			if (path[0] == _T('.')) {
				path = ::Path_AddLastSP(::GetCWD()) + path ;
				}
			}
		}
	#endif
	return	path ;
	}

////
//	*******************************************************************************
//	GetModuleFileName
//	Create	:	2021/07/06
//	Modify	:	2021/07/06
//	*******************************************************************************
inline	tstring	Get_module_name	(
#ifdef	_WIN32
	HMODULE	hModule	=	NULL
#else
	void
#endif
	)
{
	tstring	exe_name ;
	#ifdef	_WIN32
	{
		tstring	exe_temp ;
		UINT	size = MAX_PATH ;
		exe_temp.resize(size,0) ;
		::GetModuleFileName	(hModule,&exe_temp[0],size) ;
		exe_name = exe_temp.c_str() ;
		}
	#endif
	{
		v_tstring	argv = ::get_arg() ;
		if (argv.size() > 0) {
			if (!exe_name.empty()) {
				if (exe_name != argv[0]) {
					std::terr << exe_name << std::endl ;
					std::terr << argv[0]  << std::endl ;
					}
				}
			else {
				exe_name = argv[0] ;
				}
			}
		else {
			static	bool	yet = true ;
			if (yet) {
				std::terr << _T("::get_arv() ... empty") << std::endl ;
				std::terr << exe_name << std::endl ;
				yet = false ;
				}
			}
		}
	{
		exe_name = ::Get_abs_path(exe_name.c_str()) ;
		}
	return	exe_name.c_str() ;
	}
