//	*******************************************************************************
//	Name	:	openfile.hxx
//			:	file open
//	Author	:	I. Nakagawa
//	Create	:	2015/07/28
//	Modify	:	2015/07/28
//	Modify	:	2017/04/14	Add MessageBar
//	Modify	:	2024/11/22
//	*******************************************************************************

#pragma		once

#include	<iostream>
#include	"_s_func.hxx"
#include	"gettickc.hxx"

#ifdef			_DEBUG
	#ifdef		_MSC_VER
	#include	<Windows.h>
	#endif
#endif

////
//	*******************************************************************************
//	beep for debug
//	Create	:	2016/09/28
//	*******************************************************************************
inline	bool	i_Beep	(void)
{
	#ifdef	_DEBUG
	#ifdef	_MSC_VER
		::MessageBeep(-1) ;
	#else
	#endif
	#endif
	return	true ;
	}

////
//	*******************************************************************************
//	Open File
//	Create	:	2014/02/13
//	Modify	:	2016/09/28	Add "wait_sec"
//	Modify	:	2017/04/14	Add MessageBar
//	Modify	:	2017/09/20	check empty name
//	*******************************************************************************
inline	FILE*	Open_File	(LPCTSTR pathName,LPCTSTR nMode,const int wait_sec=10)
{
	if (_tcslen(pathName) == 0)	{	return	NULL ;	}
//	int	try___ms = 10 * 1000 ;				//	Try max 10 sec
	int	try___ms = wait_sec * 1000 ;		//	Try max
	int	sleep_ms = 100 ;					//	Sleep  100 ms
	int	tryCount = try___ms / sleep_ms ;
	if (tryCount <= 0)	{	tryCount = 1 ;	}
	FILE*	fp = NULL ;
	#ifdef		MessageBar
	MessageBar::ResetLevel() ;
	MessageBar	bar(_T("::Open_File ..."),tryCount) ;
	#endif
	for (int index=0 ; index<tryCount ; index++) {
		fp = ::TFOpen(pathName,nMode) ;
		if (fp != NULL)			{	break ;		}
		::Sleep_ms(sleep_ms) ;
		if (index%sleep_ms == 1) {
			::i_Beep() ;
			}
	#ifdef		MessageBar
		bar.SetBarInc() ;
	#else
		std::terr << index+1 << _T("\r") ;
	#endif
		}
	return	fp ;
	}

