//	*******************************************************************************
//	Name	:	str_chg.hxx
//			:	string change
//	Author	:	I. Nakagawa
//	Create	:	2024/11/26
//	Modify	:	2013/09/03
//			:	2014/06/12
//			:	2017/03/02	Add String_Change
//			:	2019/10/17	bug String_Change	if src_ == dst_
//			:	2021/06/08	Add String_Change
//			:	2024/11/26
//	*******************************************************************************

#pragma		once

#include	"str_uplw.hxx"

////
//	*******************************************************************************
//	Change File Separator 
//	Create	:	2013/09/03
//	Modify	:	2016/12/15	Add # %
//	*******************************************************************************
inline	tstring	String_ChangePathSP	(c_tstring& name)
{
	tstring	newName = name ;
	for (size_t index=0 ; index<newName.length() ; index++) {
		#ifdef	_MBCS
		if (  _istlead(newName[index]))	{	//	Skip 2-byte character
			index++ ;
			continue ;
			}
		#endif
		TCHAR	c = newName[index] ;
		switch	(c) {
			case	'\\' :		c = '_' ;		break ;
			case	'/' :		c = '_' ;		break ;
			case	':' :		c = '_' ;		break ;
			case	',' :		c = '.' ;		break ;
			case	';' :		c = '.' ;		break ;
			case	'*' :		c = 'x' ;		break ;
			case	'?' :		c = '_' ;		break ;
			case	'\"' :		c = '.' ;		break ;
			case	'<' :		c = '(' ;		break ;
			case	'>' :		c = ')' ;		break ;
			case	'|' :		c = '.' ;		break ;
			case	' ' :		c = '_' ;		break ;
			case	'#' :		c = '-' ;		break ;
			case	'%' :		c = '-' ;		break ;
		//	case	'&' :		c = '-' ;		break ;
		//	case	'@' :		c = '-' ;		break ;
			case	'\r' :		c = '.' ;		break ;
			case	'\n' :		c = '.' ;		break ;
			case	'\t' :		c = '.' ;		break ;
			default :			continue ;		break ;
			}
		newName[index] = c ;
		}
	return	newName ;
	}

////
//	*******************************************************************************
//	change name 7 bit
//	Create	:	2017/03/24
//	Modify	:	2017/04/21	'0'-'9' , 'A'-'Z' , 'a'-'z' , '_' , '-' , '.'
//	*******************************************************************************
inline	tstring	String_Change_7bit	(c_tstring& name)
{
	tstring	newName = name ;
	for (size_t index=0 ; index<newName.length() ; index++) {
		TCHAR	c = newName[index] ;
		c = c & 0x7f ;
		if (c < 0x20)	{		c+= 0x20 ;	}
		if (c > 0x7e)	{		c = 0x7e ;	}
		switch	(c) {
			case	' ' :		c = '_' ;		break ;			//	20
			case	'!' :		c = '1' ;		break ;			//	21
			case	'\"' :		c = '2' ;		break ;			//	22
			case	'#' :		c = '3' ;		break ;			//	23
			case	'$' :		c = '4' ;		break ;			//	24
			case	'%' :		c = '5' ;		break ;			//	25
			case	'&' :		c = '6' ;		break ;			//	26
			case	'\'' :		c = '7' ;		break ;			//	27
			case	'(' :		c = '8' ;		break ;			//	28
			case	')' :		c = '9' ;		break ;			//	29
			case	'*' :		c = 'a' ;		break ;			//	2a
			case	'+' :		c = 'b' ;		break ;			//	2b
			case	',' :		c = 'c' ;		break ;			//	2c
			case	'-' :		c = '-' ;		break ;			//	2d
			case	'.' :		c = '.' ;		break ;			//	2e
			case	'/' :		c = 'f' ;		break ;			//	2f
			case	':' :		c = 'a' ;		break ;			//	3a
			case	';' :		c = 'b' ;		break ;			//	3b
			case	'<' :		c = 'c' ;		break ;			//	3c
			case	'=' :		c = 'd' ;		break ;			//	3d
			case	'>' :		c = 'e' ;		break ;			//	3e
			case	'?' :		c = 'f' ;		break ;			//	3f
			case	'@' :		c = '0' ;		break ;			//	40
			case	'[' :		c = 'b' ;		break ;			//	5b
			case	'\\' :		c = 'c' ;		break ;			//	5c
			case	']' :		c = 'd' ;		break ;			//	5d
			case	'^' :		c = 'e' ;		break ;			//	5e
			case	'_' :		c = '_' ;		break ;			//	5f
			case	'`' :		c = '0' ;		break ;			//	60
			case	'{' :		c = 'b' ;		break ;			//	7b
			case	'|' :		c = 'c' ;		break ;			//	7c
			case	'}' :		c = 'd' ;		break ;			//	7d
			case	'~' :		c = 'e' ;		break ;			//	7e
			case	'\r' :		c = '.' ;		break ;
			case	'\n' :		c = '.' ;		break ;
			case	'\t' :		c = '.' ;		break ;
			default :							break ;
			}
		newName[index] = c ;
		}
	return	newName ;
	}

////
//	*******************************************************************************
//	Change "\r\n" -> "  "
//	Create	:	2014/06/12
//	*******************************************************************************
inline	tstring	String_CrLfToSP	(LPCTSTR str,LPCTSTR sp=_T("  "))
{
	tstring	spCR = tstring(sp).substr(0,1) ;
	tstring	spLF = tstring(sp).substr(1) ;
	tstring	tmp = str ;
	tstring	newLine ;
	for (size_t index=0 ; index<tmp.length() ; index++) {
		tstring	c = tmp.substr(index,1) ;
		if		(c == _T("\r"))	{	c = spCR ;	}
		else if	(c == _T("\n"))	{	c = spLF ;	}
		newLine += c ;
		}
	return	newLine ;
	}

inline	tstring	String_CrLfToSP	(c_tstring str,LPCTSTR sp=_T("  "))		{
	return	::String_CrLfToSP(str.c_str(),sp) ;
	}

////
//	*******************************************************************************
//	sanitize
//	Create	:	2017/05/25
//	*******************************************************************************
inline	tstring	String_sanitize	(LPCTSTR str)
{
	tstring	tmp = str ;
	tstring	newLine ;
	for (size_t index=0 ; index<tmp.length() ; index++) {
		tstring	c = tmp.substr(index,1) ;
		if		(c == _T("&"))	{	c = _T("&amp;") ;	}
		else if	(c == _T("<"))	{	c = _T("&lt;") ;	}
		else if	(c == _T(">"))	{	c = _T("&gt;") ;	}
		else if	(c == _T("\""))	{	c = _T("&quot;") ;	}
	//	else if	(c == _T("\'"))	{	c = _T("&#039;") ;	}
		newLine += c ;
		}
	return	newLine ;
	}
inline	tstring	String_sanitize	(c_tstring str)	{	return	::String_sanitize(str.c_str()) ;	}

////
//	*******************************************************************************
//	change
//	Create	:	2017/03/02
//	Modify	:	2017/04/14	bug	if src.len <  dst.len
//	Modify	:	2019/10/17	bug	if src.len <= dst.len		src_ == dst_
//	*******************************************************************************
inline	tstring	String_Change	(LPCTSTR str_,LPCTSTR src_,LPCTSTR dst_)
{
	tstring	str = str_ ;
	tstring	src = src_ ;
	tstring	dst = dst_ ;
	for (size_t index=0 ; index<str.length() ; /*index++*/) {
	//	size_t	found = str.find(src,index) ;
		size_t	found = ::String_FindNoCase(str,src,index) ;
		if (found == tstring::npos)		{	break ;		}
		tstring	pre = str.substr(0,found) ;
		tstring	pst = str.substr(found+src.length()) ;
		str = pre + dst + pst ;
		index = found ;
		if (src.length() <= dst.length()) {
			index = index + (dst.length()-src.length())+1 ;
			}
		}
	return	str ;
	}

////
//	*******************************************************************************
//	change tstring
//	Create	:	2021/06/04
//	*******************************************************************************
inline	tstring	String_Change	(c_tstring& str_,const TCHAR src,const TCHAR dst)
{
	tstring	str = str_ ;
	for (size_t index=0 ; index<str.size() ; index++) {
		TCHAR	ch = str[index] ;
		if (ch == src) {
			str[index] = dst ;
			}
		}
	return	str ;
	}





/*
#ifdef		_MSC_VER
#pragma		warning	(disable	:	4786	)
#endif
#include	<algorithm>

#include	<locale>
#include	"str_uplw.hxx"
#include	"i_define.hxx"
#include	"_tdefine.hxx"
#include	"tstring.hxx"
#include	"_s_func.hxx"
#include	"i_vector.hxx"
#include	"v_tstrng.hxx"
#include	"ttox.hxx"
#include	"t_tstrng.hxx"
#include	"tstrmbwc.hxx"

#ifndef		_istdigit
	#define	_istdigit	isdigit
#endif
#ifndef		_istspace
	#define	_istspace	isspace
#endif
#ifndef		_istalpha
	#define	_istalpha	isalpha
#endif
#ifndef		_istcntrl
	#define	_istcntrl	iscntrl
#endif

#ifdef		_DEBUG
#else
#include	"straryfc.hxx"
#endif
*/

