//	*******************************************************************************
//	Name	:	text_cc.hxx
//			:	get char code
//	Author	:	I. Nakagawa
//	Create	:	2020/01/07
//	Modify	:	2020/01/07
//	*******************************************************************************

#pragma		once

#include	<string>
#include	"def_cc.hxx"
#include	"i_define.hxx"
#include	"i_v_type.hxx"

/*
////
//	*******************************************************************************
//	define	Char Code
//	Create	:	2012/03/27
//	Modify	:	2020/01/07
//	*******************************************************************************
#define	CC_UTF_8			8
#define	CC_UTF_16			16
#define	CC_UTF_16_BE		17
#define	CC_UTF_32			32
#define	CC_UTF_32_BE		33
#define	CC_Shift_JIS		'J'
#define	CC_ASCII			'A'
#define	CC_Unknown			0
*/

////
//	*******************************************************************************
//	get char code
//	Create	:	2020/01/07
//	*******************************************************************************
//	
//		https://seiai.ed.jp/sys/text/csd/cf14/c14a070.html
//		https://seiai.ed.jp/sys/text/java/shiftjis_table.html
//	
//	UTF-8
//		0...                                    	00-7x	    	    	    	
//		110. .... 10.. ....                     	Cx-Dx	8x-bx	    	    	
//		1110 .... 10.. .... 10.. ....           	Ex  	8x-bx	8x-bx	    	
//		1111 0... 10.. .... 10.. .... 10.. .... 	F0-F7   8x-bx	8x-bx	8x-bx	
//		1111 10.. 10.. .... 10.. .... 10.. .... 	F8-FB   8x-bx	8x-bx	8x-bx	8x-bx
//		1111 110. 10.. .... 10.. .... 10.. .... 	FC-FD   8x-bx	8x-bx	8x-bx	8x-bx	8x-bx
//	
//	ShiftJIS ?    932 ?
//		0...                                    	00-7x	     
//		100. 01..                               	81-9F	40-7E
//		100. 1...                               	81-9F	80-FC
//		101.                                    	A0-Bx	 kana
//		110.                                    	C0-Dx	 kana
//		111. 01..                               	E0-FC	40-7E
//		111. 1...                               	E0-FC	80-FC
//	
//	
//	   0  1  2  3  4  5  6  7  8  9  A  B  C  D  E  F 
//	2                                                 
//	3                                                 
//	4                                                 
//	5                                                 
//	6                                                 
//	7                                                 
//	8              S  S  S  S  S  S  S  S  S  S  S  S 
//	9              S  S  S  S  S  S  S  S  S  S  S  S 
//	A                                                 
//	B                                                 
//	C                          8  8  8  8             
//	D                          8  8  8  8             
//	E              S  S  S  S  *  *  *  *  S  S  S  S 
//	F              S  S  S  S  *  *  *  *  S  S  S  S 
//	
//	Ex80 - ExBF         	Ex 8x 8x
//	Fx80 - FxBF         	Fx 8x 8x 8x
//	
//	
inline	long	get_char_code_mb	(const v_char& v_chr)
{
	long	cc = CC_Unknown ;
	{	//	check Shift JIS
		bool	only_7 = true ;
		for (size_t index=1 ; index<v_chr.size() ; index++) {
			int		m0 = u__8(v_chr[index-1]) ;
			int		m1 = u__8(v_chr[index-0]) ;
			if (m0 < 0x80)	{	continue ;		}
			only_7 = false ;
			if			(0x81 <= m0 && m0 <= 0x9f) {
				if		(0x40 <= m1 && m1 <= 0x7e) {	cc = CC_Shift_JIS ;	break ;		}
				else if	(0x80 <= m1 && m1 <= 0xfc) {	cc = CC_Shift_JIS ;	break ;		}
				}
			if			(0xe0 <= m0 && m0 <= 0xfc) {
				if		(0x40 <= m1 && m1 <= 0x7e) {	cc = CC_Shift_JIS ;	break ;		}
				else if	(0x80 <= m1 && m1 <= 0xbf) {									}	//	might be UTF-8
				else if	(0xc0 <= m1 && m1 <= 0xfc) {	cc = CC_Shift_JIS ;	break ;		}
				}
			index++ ;
			if (m1 < 0x80)	{	continue ;		}
			for ( ; index<v_chr.size() ; index++) {
				int	c = u__8(v_chr[index]) ;
				if (c < 0x80)	{	break ;		}
				}
			}
		if (only_7) {
			cc = CC_ASCII ;
			}
		}
	if (cc != CC_Unknown)	{	return	cc ;	}
	{	//	check UTF-8
		long	count_8 = 0 ;
		long	count_o = 0 ;
		for (size_t index=0 ; index<v_chr.size() ; index++) {
			int		c0 = u__8(v_chr[index]) ;
			if (c0 < 0x80)	{	continue ;		}
			int		m0 = 0 ;
			int		m1 = 0 ;
			int		m2 = 0 ;
			int		m3 = 0 ;
			int		m4 = 0 ;
			int		m5 = 0 ;
			if (index+0 < v_chr.size()) {	m0 = u__8(v_chr[index+0]) ;		}
			if (index+1 < v_chr.size()) {	m1 = u__8(v_chr[index+1]) ;		}
			if (index+2 < v_chr.size()) {	m2 = u__8(v_chr[index+2]) ;		}
			if (index+3 < v_chr.size()) {	m3 = u__8(v_chr[index+3]) ;		}
			if (index+4 < v_chr.size()) {	m4 = u__8(v_chr[index+4]) ;		}
			if (index+5 < v_chr.size()) {	m5 = u__8(v_chr[index+5]) ;		}
			if			(0xc0 <= m0 && m0 <= 0xdf) {
				if		(0x80 <= m1 && m1 <= 0xbf) {
					count_8++ ;
					index++ ;
					}
				}
			else if		(0xe0 <= m0 && m0 <= 0xef) {
				if	(	(0x80 <= m1 && m1 <= 0xbf)	&&
						(0x80 <= m2 && m2 <= 0xbf)	)	{
					count_8++ ;
					index++ ;
					index++ ;
					}
				}
			else if		(0xf0 <= m0 && m0 <= 0xf7) {
				if	(	(0x80 <= m1 && m1 <= 0xbf)	&&
						(0x80 <= m2 && m2 <= 0xbf)	&&
						(0x80 <= m3 && m3 <= 0xbf)	)	{
					count_8++ ;
					index++ ;
					index++ ;
					index++ ;
					}
				}
			else if		(0xf8 <= m0 && m0 <= 0xfb) {
				if	(	(0x80 <= m1 && m1 <= 0xbf)	&&
						(0x80 <= m2 && m2 <= 0xbf)	&&
						(0x80 <= m3 && m3 <= 0xbf)	&&
						(0x80 <= m4 && m4 <= 0xbf)	)	{
					count_8++ ;
					index++ ;
					index++ ;
					index++ ;
					index++ ;
					}
				}
			else if		(0xfc <= m0 && m0 <= 0xfd) {
				if	(	(0x80 <= m1 && m1 <= 0xbf)	&&
						(0x80 <= m2 && m2 <= 0xbf)	&&
						(0x80 <= m3 && m3 <= 0xbf)	&&
						(0x80 <= m4 && m4 <= 0xbf)	&&
						(0x80 <= m5 && m5 <= 0xbf)	)	{
					count_8++ ;
					index++ ;
					index++ ;
					index++ ;
					index++ ;
					index++ ;
					}
				}
			else {
				count_o++ ;
				}
			}
		if (count_o == 0 && count_8 > 0) {
			cc = CC_UTF_8 ;
			}
		}
	return	cc ;
	}

////
//	*******************************************************************************
//	get char code
//	Create	:	2020/01/07
//	Modify	:	2020/01/24		down "\xff\xfe"
//	*******************************************************************************
inline	long	get_char_code	(v_char* v_chr)
{
	v_char	vchr = *v_chr ;
	long	cc = CC_Unknown ;
	unsigned char	bom0 = 0xff ;	if (vchr.size()>=1)	{	bom0 = vchr[0] ;	}
	unsigned char	bom1 = 0xff ;	if (vchr.size()>=2)	{	bom1 = vchr[1] ;	}
	unsigned char	bom2 = 0xff ;	if (vchr.size()>=3)	{	bom2 = vchr[2] ;	}
	unsigned char	bom3 = 0xff ;	if (vchr.size()>=4)	{	bom3 = vchr[3] ;	}
	if		(bom0==0xff && bom1==0xfe && bom2==0x00 && bom3==0x00)	{	//	UTF-32	Little
		cc = CC_UTF_32 ;
		vchr.erase(vchr.begin(),vchr.begin()+4) ;
		vchr.push_back(0) ;
		vchr.push_back(0) ;
		vchr.push_back(0) ;
		vchr.push_back(0) ;
		}
	else if	(bom0==0x00 && bom1==0x00 && bom2==0xfe && bom3==0xff)	{	//	UTF-32	big endian
		cc = CC_UTF_32_BE ;
		vchr.erase(vchr.begin(),vchr.begin()+4) ;
		vchr.push_back(0) ;
		vchr.push_back(0) ;
		vchr.push_back(0) ;
		vchr.push_back(0) ;
		}
	else if (bom0==0xef && bom1==0xbb && bom2==0xbf)				{	//	UTF-8
		cc = CC_UTF_8 ;
		vchr.erase(vchr.begin(),vchr.begin()+3) ;
		vchr.push_back(0) ;
		}
	else if	(bom0==0xff && bom1==0xfe)								{	//	UTF-16	Little
		cc = CC_UTF_16 ;
		vchr.erase(vchr.begin(),vchr.begin()+2) ;
		vchr.push_back(0) ;
		vchr.push_back(0) ;
		}
	else if	(bom0==0xfe && bom1==0xff)								{	//	UTF-16	big endian
		cc = CC_UTF_16_BE ;
		vchr.erase(vchr.begin(),vchr.begin()+2) ;
		vchr.push_back(0) ;
		vchr.push_back(0) ;
		}
	else {
		cc = ::get_char_code_mb	(vchr) ;
		vchr.push_back(0) ;
		}
	*v_chr = vchr ;
	return	cc ;
	}
inline	long	get_char_code(c_v_char& vchr)
{
	v_char	tmp = vchr ;
	return	::get_char_code(&tmp) ;
	}

