//	*******************************************************************************
//	Name	:	xml_base.hxx
//			:	XML base
//	Author	:	I. Nakagawa
//	Create	:	2013/04/11
//	Modify	:	2013/08/08
//	Modify	:	2015/11/10	Add MessageBar
//	Modify	:	2015/11/25	Add Xml_E::ToText
//	Modify	:	2016/12/06	Change Xml_E::AddChild
//	Modify	:	2018/12/17	IsEmpty ... const	add Clear()
//	Modify	:	2019/06/25	Add GetChildElement
//	Modify	:	2024/11/27
//	*******************************************************************************

#pragma		once

#include	"i_define.hxx"
#include	"v_tstrng.hxx"
#include	"messbar.hxx"
#include	"str_join.hxx"

////
//	*******************************************************************************
//	XML Attribute
//	Create	:	2013/04/10
//	*******************************************************************************
class	Xml_Attribute	{
public:
					Xml_Attribute	()								{	;	}
					Xml_Attribute	(LPCTSTR   name,LPCTSTR   val)	{	Name = name ;	Value = val ;	}
					Xml_Attribute	(c_tstring&name,c_tstring&val)	{	Name = name ;	Value = val ;	}
public:
	virtual	bool	SetAttribute	(LPCTSTR   name,LPCTSTR   val)	{	Name = name ;	Value = val ;	return	true ;	}
	virtual	bool	SetAttribute	(c_tstring&name,c_tstring&val)	{	Name = name ;	Value = val ;	return	true ;	}
	virtual	tstring	GetName			(void)		const				{	return	Name ;	}
	virtual	tstring	GetValue		(void)		const				{	return	Value ;	}
protected:
	tstring		Name ;
	tstring		Value ;
	} ;

////
//	*******************************************************************************
//	XML Element
//	Create	:	2013/04/10
//	Modify	:	2013/08/08
//	Modify	:	2015/11/10	Add MessageBar
//	Modify	:	2015/11/24	Del copy constructor , operator=
//	Modify	:	2018/12/17	IsEmpty ... const	add Clear()
//	*******************************************************************************
class	Xml_E	{
public:
					Xml_E			(LPCTSTR   name=_T(""))			{	Name = name ;					}
					Xml_E			(LPCTSTR   name,LPCTSTR   text)	{	Name = name ;	Text = text ;	}
					Xml_E			(c_tstring&name,c_tstring&text)	{	Name = name ;	Text = text ;	}
public:
//			Xml_E					(const Xml_E& other)		{	Name = other.Name ;	Text = other.Text ;
//											Attribute = other.Attribute ;	Child = other.Child ;					}
//	virtual	Xml_E&	operator=		(const Xml_E& other)		{	Name = other.Name ;	Text = other.Text ;
//											Attribute = other.Attribute ;	Child = other.Child ;	return	*this ;	}
	virtual	bool	Clear			(void)						{	Xml_E	emp ;	*this = emp ;	return	true ;	}
public:
	virtual	bool	IsEmpty			(void)		const			{
						return			(	GetName().empty()		&&
											GetText().empty()		&&
											GetAttributeCount()	==0	&&
											GetChildCount()		==0
											) ;
										}
public:
	virtual	bool	SetName			(LPCTSTR   name)			{	Name = name ;	return	true ;	}
	virtual	bool	SetName			(c_tstring&name)			{	Name = name ;	return	true ;	}
	virtual	bool	SetText			(LPCTSTR  text)				{	Text = text ;	return	true ;	}
	virtual	bool	SetText			(c_tstring&text)			{	Text = text ;	return	true ;	}
	virtual	bool	SetComment		(c_tstring&cmt)				{	return	SetComment(cmt.c_str()) ;	}
	virtual	bool	SetComment		(LPCTSTR   cmt)				{
						SetName(_T("---"));
						tstring	comment = cmt ;
								comment = _T("\t") + comment + _T("\t") ;
						for (int index=0 ; index<int(comment.length()) ; index++) {
							int	find = int(comment.find(_T("--"),index)) ;
							if (find <0)	{	break ;		}
							comment.replace(find+1,1,_T("\t")) ;
							index = find ;
							}
						return	SetText(comment.c_str()) ;
						}
	virtual	bool	AddAttribute	(c_tstring&attN,c_tstring&value){	return	AddAttribute(attN.c_str(),value.c_str()) ;	}
	virtual	bool	AddAttribute	(LPCTSTR   attN,LPCTSTR   value){
											Attribute.push_back(Xml_Attribute(attN,value)) ;		return	true ;	}
public:
	virtual	tstring	GetName			(void)		const				{	return	Name ;	}
	virtual	tstring	GetText			(void)		const				{	return	Text ;	}
	virtual	bool	IsComment		(void)		const				{	return	(Name==_T("---")) ;			}
	virtual	int	GetAttributeCount	(void)		const				{	return	int(Attribute.size()) ;		}
	virtual	Xml_Attribute
					GetAttribute	(const long index)	const	{
										Xml_Attribute	error ;
										if (index < 0)						{	return	error ;	}
										if (GetAttributeCount() <= index)	{	return	error ;	}
										return	Attribute[index] ;
										}
public:
	virtual	bool	AddChild		(const Xml_E& ch) ;
	virtual	int		GetChildCount	(void)		const				{	return	int(Child.size()) ;			}
	virtual	Xml_E	GetChild		(const long index)	const	{
										Xml_E	error ;
										if (index < 0)						{	return	error ;	}
										if (GetChildCount() <= index)		{	return	error ;	}
										return	Child[index] ;
										}
public:
	virtual	tstring	GetAttribute	(c_tstring&attrName)		const	{	return	GetAttribute(attrName.c_str()) ;	}
	virtual	tstring	GetAttribute	(LPCTSTR   attrName)		const	{
										tstring	value ;
										for (int aIndex=0 ; aIndex<GetAttributeCount() ; aIndex++) {
											Xml_Attribute	a = GetAttribute(aIndex) ;
											tstring	aName = a.GetName () ;
											if (attrName == aName) {
												value = a.GetValue() ;
												break ;
												}
											}
										return	value ;
										}
	virtual	Xml_E	GetChildElement	(c_tstring&name)			const	{	return	GetChildElement(name.c_str()) ;	}
	virtual	Xml_E	GetChildElement	(LPCTSTR   name)			const	{
										MessageBar	bar(_T("Xml_E::GetElement ..."),GetChildCount()) ;
										for (int cIndex=0 ; cIndex<GetChildCount() ; cIndex++) {
											bar.SetBarInc() ;
											Xml_E	c = GetChild(cIndex) ;
											Xml_E	e = c.GetElement(name) ;
											if (e.GetName() == name)	{
												return	e ;
												}
											}
										Xml_E	error ;
										return	error ;
										}
	virtual	Xml_E	GetElement		(c_tstring&name)			const	{	return	GetElement(name.c_str()) ;	}
	virtual	Xml_E	GetElement		(LPCTSTR   name)			const	{
										if (GetName() == name)	{
											return	*this ;
											}
										/*
										MessageBar	bar(_T("Xml_E::GetElement ..."),GetChildCount()) ;
										for (int cIndex=0 ; cIndex<GetChildCount() ; cIndex++) {
											bar.SetBarInc() ;
											Xml_E	c = GetChild(cIndex) ;
											Xml_E	e = c.GetElement(name) ;
											if (e.GetName() == name)	{
												return	e ;
												}
											}
										Xml_E	error ;
										return	error ;
										*/
										return	GetChildElement(name) ;
										}
public:
	virtual	tstring	ToText			(void)		const ;
protected:
	virtual	tstring	Child_ToText	(void)		const ;
protected:
	tstring						Name ;
	tstring						Text ;
	std::vector<Xml_Attribute>	Attribute ;
protected:
	std::vector<Xml_E>			Child ;
	} ;

////
//	*******************************************************************************
//	AddChild
//	Create	:	2013/04/10
//	Modify	:	2016/12/06
//	*******************************************************************************
//inline	bool	Xml_E::AddChild		(const Xml_E& ch)	{	Child.push_back(ch);		return	true ;	}
inline		bool	Xml_E::AddChild		(const Xml_E& ch)
{
	if (!ch.GetName().empty()) {
		Child.push_back(ch) ;
		}
	else {
		for (int index=0 ; index<ch.GetChildCount() ; index++) {
			Xml_E	ch1 = ch.GetChild(index) ;
			this->AddChild(ch1) ;
			}
		}
	return	true ;
	}

////
//	*******************************************************************************
//	ToText
//	Create	:	2015/11/26
//	Modify	:	2017/02/03	Add child when empty
//	*******************************************************************************
inline	tstring	Xml_E::ToText	(void)	const
{
	if (this->GetName().empty() && this->GetText().empty()) {
		return	this->Child_ToText() ;
		}
	if (this->GetName().empty()) {
		return	this->GetText() ;
		}
	if (this->IsComment()) {
		tstring	cmtStr ;
		{
			cmtStr +=	_T("<!--") +	this->GetText()	+	_T("-->") ;
			if (this->GetChildCount() > 0) {
				cmtStr +=	_T("\r\n") ;
				cmtStr +=	_T("<!--") ;
				{
					cmtStr += this->Child_ToText() ;
					}
				cmtStr +=	_T("-->") ;
				}
			}
		return	cmtStr ;
		}
	tstring	eleStr ;
	{											//	Start tag
		eleStr = _T("<") + this->GetName() + _T("\t") ;
		tstring	attStr ;						//	Attribute
		for (int aIndex=0 ; aIndex<this->GetAttributeCount() ; aIndex++) {
			Xml_Attribute	a = this->GetAttribute(aIndex) ;
			tstring	name = a.GetName ().c_str() ;
			tstring	value= a.GetValue().c_str() ;
			attStr += name + _T("=\"") + value + _T("\"\t") ;
			}
		if (!attStr.empty()) {
			eleStr += attStr ;
			}
		bool	existChild = false ;
		bool	existText  = false ;
		tstring	nameOrChild ;
		if (!this->GetText().empty()) {	//	text
			tstring	strText = this->GetText().c_str() ;
			eleStr += _T(">") + strText ;
			existText = true ;
			}
		{
			tstring	strChild = this->Child_ToText() ;
			if (!strChild.empty()) {
				if (existText)	{	eleStr += _T( "\t\t") + strChild ;		}
				else			{	eleStr += _T(">\t\t") + strChild ;		}
				existChild = true ;
				}
			}
		if (!existChild && !existText)	{		//	child , text  is empty ?
			eleStr += _T("/>") ;				//	<tag attr1='1' ... attrn='n' />
			}
		else {									//	End tag
			if (existChild) {					//	<tag attr1='1' ... > ... </tag>
				eleStr +=	tstring(_T("\t"))	+	_T("</") + this->GetName()	+	_T(">") ;
				}
			else {								//	<tag attr1='1' ... >.....</tag>
				eleStr += 	tstring(_T(""))		+	_T("</") + this->GetName()	+	_T(">") ;
				}
			}
		}
	return	eleStr ;
	}

////
//	*******************************************************************************
//	Child ToText
//	Create	:	2015/11/26
//	*******************************************************************************
inline	tstring	Xml_E::Child_ToText	(void)	const
{
	tstring		strChild ;
	if (this->GetChildCount() > 0) {
		v_tstring	saChild ;
		for (int cIndex=0 ; cIndex<this->GetChildCount() ; cIndex++) {
			Xml_E		c = this->GetChild(cIndex) ;
			tstring		sc = c.ToText() ;
			saChild.push_back(sc) ;
			}
		strChild = ::String_Join(saChild,_T("\t")) ;
		}
	return	strChild ;
	}

