//	*******************************************************************************
//	Name	:	xml_css.hxx
//			:	CSS Base
//	Author	:	I. Nakagawa
//	Create	:	2017/05/25
//	Modify	:	2025/06/03
//	*******************************************************************************

#pragma		once

#include	"str_trim.hxx"

////
//	*******************************************************************************
//	XML Style
//	Create	:	2017/05/25
//	*******************************************************************************
class	Xml_Style	{
public:
					Xml_Style		()								{	;	}
					Xml_Style		(LPCTSTR   name,LPCTSTR   val)	{	Name = name ;	Value = val ;	}
					Xml_Style		(c_tstring&name,c_tstring&val)	{	Name = name ;	Value = val ;	}
public:
	virtual	bool	SetStyle		(LPCTSTR   name,LPCTSTR   val)	{	Name = name ;	Value = val ;	return	true ;	}
	virtual	bool	SetStyle		(c_tstring&name,c_tstring&val)	{	Name = name ;	Value = val ;	return	true ;	}
	virtual	tstring	GetName			(void)		const				{	return	Name ;	}
	virtual	tstring	GetValue		(void)		const				{	return	Value ;	}
protected:
	tstring		Name ;
	tstring		Value ;
	} ;

//	typedef	std::vector<Xml_Style>	v_Xml_Style ;
class	v_Xml_Style	{
public:
	virtual	bool	AddStyle		(c_tstring&attN,c_tstring&value){	return	AddStyle(attN.c_str(),value.c_str()) ;	}
	virtual	bool	AddStyle		(LPCTSTR   attN,LPCTSTR   value){
											v_Style.push_back(Xml_Style(attN,value)) ;		return	true ;	}
public:
	virtual	long	GetCount			(void)		const				{	return	long(v_Style.size()) ;		}
	virtual	Xml_Style
					GetStyle		(const long index)	const	{
										Xml_Style	error ;
										if (index < 0)						{	return	error ;	}
										if (GetCount() <= index)			{	return	error ;	}
										return	v_Style[index] ;
										}
public:
	virtual	tstring	ToText			(void)		const	{
										tstring	text ;
										for (long index=0 ; index<GetCount() ; index++) {
											Xml_Style	s = GetStyle(index) ;
											tstring	name = s.GetName ().c_str() ;
											tstring	value= s.GetValue().c_str() ;
											if (name. empty())	{	continue ;	}
											if (value.empty())	{	continue ;	}
											text += name + _T(" : ") + value + _T(" ;\t") ;
											}
										text = ::String_TrimBoth(text) ;
										return	text ;
										}
protected:
	std::vector<Xml_Style>		v_Style ;
	} ;

/*
////
//	*******************************************************************************
//	ToText
//	Create	:	2017/05/25
//	*******************************************************************************
inline	tstring	v_Xml_Style::ToText	(void)	const
{
	tstring	text ;
	for (size_t index=0 ; index<styleAry.GetCount() ; index++) {
		Xml_Style	s = styleAry.GetStyle(index) ;
		tstring	name = s.GetName ().c_str() ;
		tstring	value= s.GetValue().c_str() ;
		if (name. empty())	{	continue ;	}
		if (value.empty())	{	continue ;	}
		text += name + _T(" : ") + value + _T(";\t") ;
		}
	text = ::String_Trim(text) ;
	return	text ;
	}
*/

#define	CSS_text_align		_T("text-align")
#define	CSS_center			_T("center")
#define	CSS_right			_T("right")
#define	CSS_white_space		_T("white-space")
#define	CSS_nowrap			_T("nowrap")

#define	CSS_margin_left		_T("margin-left")
#define	CSS_margin_right	_T("margin-right")
#define	CSS_auto			_T("auto")

////
//	*******************************************************************************
//	text-align
//	white-space
//	Create	:	2017/05/25
//	*******************************************************************************
inline	tstring	CSS_text_align_center	(void)	{	v_Xml_Style	ta ;	ta.AddStyle(CSS_text_align,CSS_center);		return	ta.ToText() ;		}
inline	tstring	CSS_text_align_right	(void)	{	v_Xml_Style	ta ;	ta.AddStyle(CSS_text_align,CSS_right) ;		return	ta.ToText() ;		}
inline	tstring	CSS_white_space_nowrap	(void)	{	v_Xml_Style	ws ;	ws.AddStyle(CSS_white_space,CSS_nowrap);	return	ws.ToText() ;		}

////
//	*******************************************************************************
//	margin-left : auto ; margin-right : auto ;
//	Create	:	2017/07/04
//	*******************************************************************************
inline	tstring	CSS_margin_auto			(void)	{	v_Xml_Style	mg ;	mg.AddStyle(CSS_margin_left, CSS_auto);
																		mg.AddStyle(CSS_margin_right,CSS_auto);		return	mg.ToText() ;		}
